@@ -12,7 +12,7 @@ use Monitoring::TT::Object;
 use Monitoring::TT::Render;
 use Monitoring::TT::Utils;
 
-our $VERSION = '1.0.0';
+our $VERSION = '1.0.1';
 
 #####################################################################
 
@@ -223,25 +223,25 @@ sub _build_dynamic_config {
 sub _build_dynamic_object_config {
     my($self) = @_;
 
-    # build templates
-    my $templates = {
-        contacts => $self->_build_template('conf.d', 'contacts'),
-        hosts    => $self->_build_template('conf.d', 'hosts', [ 'conf.d/apps', 'conf.d/apps.cfg' ]),
-    };
-
     # detect input type
     my $input_types = $self->_get_input_types($self->{'in'});
 
     # no dynamic config at all?
     return unless scalar keys %{$input_types} > 0;
 
+    # build templates
+    my $templates = {
+        contacts => $self->_build_template('conf.d', 'contacts'),
+        hosts    => $self->_build_template('conf.d', 'hosts', [ 'conf.d/apps', 'conf.d/apps.cfg' ]),
+    };
+
     mkdir($self->{'out'}.'/conf.d');
 
     my $data = { hosts => [], contacts => []};
     for my $type (keys %{$input_types}) {
         my $typefilter = $self->{'opt'}->{substr($type,0,-1).'filter'};
         my $obj_list = [];
-        trace('fetching data for '.$type);
+        trace('fetching data for '.$type) if $Monitoring::TT::Log::Verbose >= 4;
         for my $cls (@{$input_types->{$type}}) {
             for my $in (@{$self->{'in'}}) {
                 my $data = $cls->read($in, $type);
@@ -329,7 +329,7 @@ sub _build_template {
         for my $path (glob($in.'/'.$dir.'/'.$type.'/ '.
                            $in.'/'.$dir.'/'.$type.'*.cfg')
         ) {
-            trace('looking for '.$type.' templates in '.$path);
+            trace('looking for '.$type.' templates in '.$path) if $Monitoring::TT::Log::Verbose >= 4;
             if(-e $path) {
                 my $templates = $self->_get_files($path, '\.cfg');
                 for my $t (reverse sort @{$templates}) {
@@ -362,7 +362,7 @@ sub _build_template {
         for my $in (@{$self->{'in'}}) {
             for my $path (@{$appdirs}) {
                 $path = $in.'/'.$path;
-                trace('looking for '.$type.' apps in '.$path);
+                trace('looking for '.$type.' apps in '.$path) if $Monitoring::TT::Log::Verbose >= 4;
                 if(-e $path) {
                     my $templates = $self->_get_files($path, '\.cfg');
                     for my $t (reverse sort @{$templates}) {
@@ -390,8 +390,10 @@ sub _build_template {
         return '';
     }
     $template .= "[% END %]\n";
-    trace('created template:');
-    trace($template);
+    if($Monitoring::TT::Log::Verbose >= 4) {
+        trace('created template:');
+        trace($template);
+    }
     return $template;
 }
 
@@ -420,25 +422,31 @@ sub _get_files {
 sub _process_template {
     my($self, $template, $data) = @_;
 
-    for my $in (@{$self->{'in'}}) {
-        debug('looking for a '.$in.'/config.cfg');
-        if(-e $in.'/config.cfg') {
-            debug('added config template '.$in.'/config.cfg');
-            $template = $self->_read_replaced_template($in.'/config.cfg')."\n".$template;
+    if(!defined $self->{'_config_template'}) {
+        $self->{'_config_template'} = "";
+        for my $in (@{$self->{'in'}}) {
+            debug('looking for a '.$in.'/config.cfg');
+            if(-e $in.'/config.cfg') {
+                debug('added config template '.$in.'/config.cfg');
+                $self->{'_config_template'} .= $self->_read_replaced_template($in.'/config.cfg')."\n";
+            }
         }
     }
+    $template = $self->{'_config_template'}.$template;
 
-    trace('template:');
-    trace('==========================');
-    trace($template);
-    trace('==========================');
+    if($Monitoring::TT::Log::Verbose >= 4) {
+        trace('template:');
+        trace('==========================');
+        trace($template);
+        trace('==========================');
+    }
 
     my $output;
     $self->tt->process(\$template, $data, \$output) or $self->_template_process_die($template, $data);
 
     # clean up result
-    $output =~ s/^\s*$//sgmx;
-    $output =~ s/^\n//gmx;
+    $output =~ s/^\s*$//sgmxo;
+    $output =~ s/^\n//gmxo;
 
     return $output;
 }
@@ -451,11 +459,13 @@ sub _get_input_classes {
     for my $dir (@{$folders}) {
         next unless -d $dir.'/lib/.';
         unshift @INC, "$dir/lib";
-        trace('added '.$dir.'/lib to @INC');
+        trace('added '.$dir.'/lib to @INC') if $Monitoring::TT::Log::Verbose >= 4;
     }
 
-    trace('@INC:');
-    trace(\@INC);
+    if($Monitoring::TT::Log::Verbose >= 4) {
+        trace('@INC:');
+        trace(\@INC);
+    }
 
     my $uniq_types = {};
     my $uniq_libs  = {};
@@ -464,7 +474,7 @@ sub _get_input_classes {
         $uniq_libs->{$inc} = 1;
         my @files = glob($inc.'/Monitoring/TT/Input/*.pm');
         for my $file (glob($inc.'/Monitoring/TT/Input/*.pm')) {
-            trace('found input class: '.$file);
+            trace('found input class: '.$file) if $Monitoring::TT::Log::Verbose >= 4;
             $file =~ s|^$inc/Monitoring/TT/Input/||mx;
             $file =~ s|\.pm$||mx;
             push @{$types}, $file unless defined $uniq_types->{$types}->{$file};
@@ -489,7 +499,7 @@ sub _get_input_types {
         my $obj      = \&{$objclass."::new"};
         my $it       = &$obj($objclass, montt => $self);
         my $types    = $it->get_types($folders);
-        trace('input \''.$t.'\' supports: '.join(', ', @{$types}));
+        trace('input \''.$t.'\' supports: '.join(', ', @{$types})) if $Monitoring::TT::Log::Verbose >= 4;
         for my $type (@{$types}) {
             $input_types->{$type} = [] unless defined $input_types->{$type};
             push @{$input_types->{$type}}, $it;
@@ -504,7 +514,7 @@ sub _run_hook {
     return if $self->{'opt'}->{'dryrun'};
     for my $in (@{$self->{'in'}}) {
         my $hook = $in.'/hooks/'.$name;
-        trace("hook: looking for ".$hook);
+        trace("hook: looking for ".$hook) if $Monitoring::TT::Log::Verbose >= 4;
         if(-x $hook) {
             my $cmd = $hook;
             $cmd = $cmd." ".$args if defined $args;
@@ -528,7 +538,7 @@ sub _run_hook {
 #####################################################################
 sub _read_replaced_template {
     my($self, $template) = @_;
-    $template =~ s|//|/|gmx;
+    $template =~ s|//|/|gmxo;
     my $text = '[%# SRC '.$template.':1 #%]';
     open(my $fh, '<', $template) or die("cannot read: ".$template.': '.$!);
     while(my $line = <$fh>) {
@@ -542,9 +552,9 @@ sub _read_replaced_template {
         $text .= $line;
         if($line =~ m/^define\s+(\w+)/mxo) {
             if($1 eq 'service' or $1 eq 'host' or $1 eq 'contact') {
-                $text .= "  _SRC ".$template.':'.$.."\n";
+                $text .= '  _SRC '.$template.':'.$.."\n";
             } else {
-                $text .= "# SRC ".$template.':'.$.."\n";
+                $text .= '# SRC '.$template.':'.$.."\n";
             }
         }
     }
@@ -601,7 +611,7 @@ sub _template_process_die {
     error($tterror) unless $already_printed;
     debug('in template:');
     debug($template);
-    trace($data);
+    trace($data) if $Monitoring::TT::Log::Verbose >= 4;
     exit 1;
 }
 
